﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

/// <summary>
/// Claim Data Access Object
/// </summary>
public class ClaimDAO
{
    public String MyConnString { set; get; }
    public ClaimDAO()
    {
        MyConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString;
    }
    public List<Claim> getClaimList()
    {
        return getClaimList(null);
    }
    /* fetch list of claims which have not yet been approved */
    public List<Claim> getClaimList(String whereClause)
    {
        if (null == whereClause)
        {
            whereClause = "";
        }
        List<Claim> foundClaims = new List<Claim>();
        SqlConnection myConn = new SqlConnection(MyConnString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM claim " + whereClause;
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        while (myDR.Read())
        {
            /* create new DTO and add to list */
            foundClaims.Add(makeDTO(myDR));
        }
        myConn.Close();
        return foundClaims;
    }
    /* fetch a claim by ID or key */
    public Claim getClaim(int claimKey)
    {
        SqlConnection myConn = new SqlConnection(MyConnString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM claim WHERE (claimID=";
        mySelectStatement += claimKey.ToString() + ")";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return null;
        return makeDTO(myDR);
    }
    /* fetch a claim by reference number (the first one to match) */
    public Claim getClaimByRef(String claimRef)
    {
        SqlConnection myConn = new SqlConnection(MyConnString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM claim WHERE (reference='";
        mySelectStatement += claimRef + "')";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return null;
        Claim myClaim = makeDTO(myDR);
        return myClaim;
    }
    /* helper method */
    private Claim makeDTO(SqlDataReader myDR)
    {
        Claim myClaim = new Claim();
        myClaim.ClaimID = myDR.GetInt32(0);
        myClaim.Reference = myDR.GetString(1);
        myClaim.Description = myDR.GetString(2);
        myClaim.Amount = myDR.GetDecimal(3);
        myClaim.IsApproved = (myDR.GetString(4).StartsWith("Y"));
        myClaim.DateOfClaim = myDR.GetDateTime(5);
        myClaim.PolicyID = myDR.GetInt32(6);
        return myClaim;
    }
    /* save a claim object to the database */
    public void saveClaim(Claim myClaim)
    {
        char isApproved = 'N';
        if (myClaim.IsApproved == true)
        {
            isApproved = 'Y';
        }
        String claimDate = myClaim.DateOfClaim.Day + "/" + myClaim.DateOfClaim.Month + "/" + myClaim.DateOfClaim.Year;
        SqlConnection myConn = new SqlConnection(MyConnString);
        myConn.Open();
        string insertString = "insert into claim (reference, description, amount, approved, claimDate, policyID) " +
            "values (@reference, @description, @amount, @approved, @claimDate, @policyID)";
        SqlCommand myComm = new SqlCommand(insertString, myConn);

        myComm.Parameters.Add("@reference", SqlDbType.NVarChar).Value = myClaim.Reference;
        myComm.Parameters.Add("@description", SqlDbType.NVarChar).Value = myClaim.Description;
        myComm.Parameters.Add("@amount", SqlDbType.Decimal).Value = myClaim.Amount;
        myComm.Parameters.Add("@approved", SqlDbType.NVarChar).Value = isApproved;
        myComm.Parameters.Add("@claimDate", SqlDbType.DateTime).Value = claimDate;
        myComm.Parameters.Add("@policyID", SqlDbType.Int).Value = myClaim.PolicyID;

        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Insert");
    }

    /* update claim object in the database */
    public void updateClaim(Claim myClaim)
    {
        SqlConnection myConn = new SqlConnection(MyConnString);
        myConn.Open();
        SqlCommand myComm = new SqlCommand("UPDATE claim SET reference = @reference, description = @description, amount = @amount, approved = @approved, claimDate = @claimDate, policyID = @policyID WHERE claimID = @claimID",
            myConn);
        myComm.Parameters.AddWithValue("@claimID", myClaim.ClaimID);
        myComm.Parameters.AddWithValue("@reference", myClaim.Reference);
        myComm.Parameters.AddWithValue("@description", myClaim.Description);
        myComm.Parameters.AddWithValue("@amount", myClaim.Amount);
        myComm.Parameters.AddWithValue("@approved", (myClaim.IsApproved ? "Y" : "N"));
        myComm.Parameters.AddWithValue("@claimDate", myClaim.DateOfClaim);
        myComm.Parameters.AddWithValue("@policyID", myClaim.PolicyID);
        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Update");
    }
}
