﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

/// <summary>
/// Policy Data Access Object
/// </summary>
public class PolicyDAO
{
    public string connectionString { set; get; }

    public PolicyDAO()
    {
        connectionString = ConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString;
    }
    public List<Policy> getPolicyList()
    {
        return getPolicyList(null);
    }

    public List<Policy> getPolicyList(String whereClause)
    {
        if (null == whereClause)
        {
            whereClause = "";
        }
        List<Policy> foundPolicies = new List<Policy>();
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM policy " + whereClause;
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        while (myDR.Read())
        {
            foundPolicies.Add(makeDTO(myDR));
            /* create new DTO and add to list */
        }
        myConn.Close();
        return foundPolicies;
    }
    /* fetch a policy by ID or key */
    public Policy getPolicyByKey(int policyKey)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM policy WHERE (policyID=";
        mySelectStatement += policyKey.ToString() + ")";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return null;
        myConn.Close();
        return makeDTO(myDR);
    }
    /* fetch a policy by policyNumber */
    public Policy getPolicyByNumber(int policyNumber)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM policy WHERE (policyNumber=";
        mySelectStatement += policyNumber.ToString() + ")";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return null;
        Policy myPolicy = makeDTO(myDR);
        myConn.Close();
        return myPolicy;
    }
    /* helper method */
    private Policy makeDTO(SqlDataReader myDR)
    {
        Policy myPolicy = new Policy();
        myPolicy.PolicyID = myDR.GetInt32(0);
        myPolicy.Number = myDR.GetInt32(1);
        myPolicy.Type = myDR.GetString(2);
        myPolicy.StartDate = myDR.GetDateTime(3);
        myPolicy.AnnualPremium = myDR.GetDecimal(4);
        myPolicy.IsPaidUp = (myDR.GetString(5).StartsWith("Y"));
        myPolicy.PolicyHolderID = myDR.GetInt32(6);
        return myPolicy;
    }
    /* save a policy object to the database */
    public void savePolicy(Policy myPolicy)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        SqlCommand myComm = new SqlCommand("UPDATE policy SET reference = @reference, description = @description, amount = @amount, approved = @approved, claimDate = @claimDate, policyID = @policyID WHERE claimID = @claimID",
            myConn);
        myComm.Parameters.AddWithValue("@policyID", myPolicy.PolicyID);
        myComm.Parameters.AddWithValue("@policyNumber", myPolicy.Number);
        myComm.Parameters.AddWithValue("@policyType", myPolicy.Type);
        myComm.Parameters.AddWithValue("@startDate", myPolicy.StartDate);
        myComm.Parameters.AddWithValue("@annualPremium", myPolicy.AnnualPremium);
        myComm.Parameters.AddWithValue("@isPaidUp", (myPolicy.IsPaidUp ? "Y" : "N"));
        myComm.Parameters.AddWithValue("@policyHolderID", myPolicy.PolicyID);
        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Update");
    }


}
